/* LaTeX2Sdoc
 *  Copyright (C) 2000-2001 SAKURAI, Masashi (m.sakurai@cmt.phys.kyushu-u.ac.jp)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package jp.gr.java_conf.ccs2.tool.tex2sdoc.math;
import jp.gr.java_conf.ccs2.tool.tex2sdoc.math.handler.*;
import java.util.*;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import jp.gr.java_conf.ccs2.jdk11.util.StringUtil;
import jp.gr.java_conf.ccs2.jdk11.design.Singleton;
import jp.gr.java_conf.ccs2.tool.tex2sdoc.TeXUtil;
import jp.gr.java_conf.ccs2.tool.tex2sdoc.ArgumentInfo;

/**
   MathMLCommand handler manager
 */
public class MathMLCommandTable {

    static Hashtable table;//cashe
    static boolean debug = false;

    public static MathMLCommandHandler getHandler(String cmdName) {
	if (table == null) {
	    table = new Hashtable();
	}
	MathMLCommandHandler gen = 
	    (MathMLCommandHandler)table.get(cmdName);
	if (gen == null) {
	    gen = makeInstance(cmdName);
	    if (gen == null) return null;
	}
	return gen;
    }

    public static MathMLCommandHandler getHandler(CommandContent cmd) {
	if (table == null) {
	    table = new Hashtable();
	}
	String cmdName = cmd.getContent();
	MathMLCommandHandler gen = 
	    (MathMLCommandHandler)table.get(cmdName);
	if (gen == null) {
	    gen = makeInstance(cmdName);
	    if (gen == null) return null;
	}
	return gen.getCopy(cmd);
    }

    static MathMLCommandHandler makeInstance(String cmdName) {
	if (resources == null && !loadFailed) {
	    loadResources();
	}
	try {
	    return (MathMLCommandHandler)table.get(cmdName);
	} catch (MissingResourceException e) {
	    System.err.println("Error! no command definition : "+cmdName);
	    e.printStackTrace();
	} catch (Exception e) {
	    e.printStackTrace();
	}
	return null;
    }

    /**
       parse resource string

       Ex1.   \left(  =>  <mo>(</mo>  
          [throwing away "\left" command]
	  ResourceString : left=*

       Ex2.   \quad  =>  <mspace width="10pt"/>
          [simple replacing with attribute]
	  ResourceString : quad=mspace@width=\"10pt\"

       Ex3.   \sqrt{ a } => <msqrt> <mi>a</mi> </msqrt>
          [one argument command with no argument-fence tag]
	  ResourceString : sqrt=msqrt,

       Ex4.   \frac{ 1 }{ 2 }  =>  
                  <mfrac> <mrow><mn>1</mi> <mn>2</mn> </mfrac>
	  [alias names, two argument command and argument-fence tag, mrow]
	  ResourceString : frac|cfrac|dfrac=mfrac,mrow,mrow

       Ex5.   {\displaystyle A B } => 
                <mrow>
                  <mstyle displaystyle="true" scriptlevel="0">
		    A B
		  </mstyle>
	        </mrow>
	  [style tag, and interrupt end-tag]
	  ResourceString : displaystyle=#mstyle@displaystyle=\"true\" scriptlevel=\"0\"

       Ex6.   \hat{ a } => <mover> <mi>a</mi> <mo>&hat;</mo> </mover>
          [one argument command and make accent character (&hat;)]
	  ResourceString : hat=^&hat;

       Ex7.   \stackover{ f }{ \to }  => 
                                   <mover>
				     <mo>&to;</mo> <mi>f</mi>
				   </mover>
	  [arrange indexies of arguments]
	  ResourceString : stackover={10}mover,,

       Ex8.   \binom{ a }{ b } => 
                     <mo>(</mo> <mfrac linethickness="0">
		         <mi>a</mi> <mi>b</mi>
		     </mfrac> <mo>)</mo>
          [append something at pre and post tag]
	  ResourceString : 
	  binom=[<mo>(</mo>][<mo>)</mo>]mfrac@linethickness=\"0\",mrow,mrow

     */
    static void parseResource() {
	Enumeration e = resources.getKeys();
	while(e.hasMoreElements()){
	    String name = (String) e.nextElement();
	    try {
		String data = resources.getString(name);
		parseCommand(name,data);
	    } catch (MissingResourceException f) {
		throw new InternalError("TeX2MathML bug. (failed in parsing tex2mathml handler properties.)");
	    }
	}
    }

    static void parseCommand(String cmdName,String text) {
	if (text == null || text.length() == 0) return;

	MathMLCommandTable_LineParser obj = 
	    new MathMLCommandTable_LineParser(cmdName,text);

	if (obj == null) {
	    throw new InternalError(
		"Can not build CommandHandler. ["+cmdName+"] from ["+text+"]");
	}

	//debug
	if (debug) {
	    System.out.println("--------------------");
	    System.out.println("CMD:"+cmdName);
	    System.out.println(obj.getHandler());
	}

	// parse alias name
	int als = cmdName.indexOf('|');
	if (als == -1) {
	    //sigle name
	    addTable(cmdName,obj.getHandler());
	    return;
	} else {
	    //some alias
	    String [] cmdNames = StringUtil.split(cmdName,"|");
	    for (int i=0;i<cmdNames.length;i++) {
		addTable(cmdNames[i],obj.getHandler());
	    }
	}
	return;
    }

    static void addTable(String name,Object obj) {
	if (table == null) 
	    table = new Hashtable();
	table.put(name,obj);
    }

    protected static ResourceBundle resources;
    private static boolean loadFailed = false;

    static void loadResources() {
	try {
	    resources = ResourceBundle.getBundle(
		"jp.gr.java_conf.ccs2.tool.tex2sdoc.math.MathMLCommandHandler");
	    parseResource();
	} catch (MissingResourceException mre) {
	    System.err.println("[MathMLCommandHandler.properties] not found");
	    loadFailed = true;
	}
    }

    public static void main(String [] args) {
	//check
	debug = true;
	loadResources();
    }
}

class MathMLCommandTable_LineParser {

    String cmdName;
    MathMLCommandHandler handler;

    MathMLCommandTable_LineParser(String cmdName,String input) {
	this.cmdName = cmdName;
	init(input);
    }

    MathMLCommandHandler getHandler() {
	return handler;
    }
    
    void init(String input) {
	String [] tags = StringUtil.split(input,",");
	if (tags == null || tags.length == 0) {
	    error();
	}

	if (tags[0].charAt(0)=='*') {
	    //ignore
	    handler = new IgnoreCommandHandler(cmdName);
	    return;
	}
	if (tags[0].charAt(0)=='%') {
	    //native
	    loadNativeClass(tags[0].substring(1));
	    return;
	}
	if (tags[0].charAt(0)=='#') {
	    //interrupted
	    buildInterruptTag(tags[0].substring(1));
	    return;
	}
	if (tags[0].charAt(0)=='^') {
	    //accent
	    buildAccentTag(tags[0].substring(1),tags[1]);
	    return;
	}

	String [] args = new String[tags.length-1];
	for (int i=0;i<(tags.length-1);i++) {
	    args[i] = tags[i+1];
	}

	if (tags[0].charAt(0)=='!') {
	    //text
	    buildTextTag(tags[0].substring(1),args);
	    return;
	}

	int [] arrange = null;
	if (tags[0].charAt(0) == '{') {
	    //arrange index
	    ArgumentInfo info = TeXUtil.parseMatchedBrace(tags[0], 0,1);
	    if (info == null) {
		System.err.println("wrong arrangement");
		error();
	    }
	    String it = info.contents[0];
	    tags[0] = tags[0].substring(info.endPos);
	    arrange = new int[it.length()];
	    try {
		for(int i=0;i<it.length();i++) {
		    arrange[i] = Integer.parseInt(it.substring(i,i+1));
		}
	    } catch (NumberFormatException e) {
		System.err.println("wrong arrangement number." );
		error();
	    }
	}
	String pre=null,post=null;
	if (tags[0].charAt(0) == '[') {
	    //arrange index
	    ArgumentInfo info = TeXUtil.parseMatchedBrace(tags[0],'[',']',0,2);
	    if (info == null) {
		System.err.println("wrong arrangement");
		error();
	    }
	    pre = info.contents[0];
	    post = info.contents[1];
	    tags[0] = tags[0].substring(info.endPos);
	}

	//normal
	buildNormalTag(tags[0],args,arrange,pre,post);
    }

    void loadNativeClass(String cname) {
	try {
	    Class cls = Class.forName(cname);
	    handler = (MathMLCommandHandler)cls.newInstance();
	} catch (InstantiationException e) {
	    System.err.println("InstantiationException ["+
			       e.getMessage()+"]");
	} catch (IllegalAccessException e) {
	    System.err.println("IllegalAccessException ["+
			       e.getMessage()+"]");
	} catch (ClassNotFoundException e) {
	    System.err.println("can not load the class ["+cname+"]");
	}
    }

    void buildNormalTag(String cmdTag,String [] cmdArgs,int [] ar,
			String pre,String post) {
	handler = new SimpleCommandHandler(
	    cmdTag.toString(),cmdArgs,ar,pre,post);
    }

    void buildTextTag(String cmdTag,String [] cmdArgs) {
	handler = new TextHandler(
	    cmdTag.toString(),cmdArgs);
    }

    void buildAccentTag(String tag,String accentChar) {
	handler = new AccentHandler(tag, accentChar );
    }

    void buildInterruptTag(String cmdTag) {
	handler = new SimpleTemporaryCommandHandler(cmdTag);
    }

    void error() {
	throw new InternalError("Error ["+cmdName+"] line in MathMLCommandHandler.properties");
    }
}
